# This script will plot temperature and trip graph from the thermald debug output
# Run thermald with loglevel=debug
# For example:
#
# systemctl stop thermald
# thermald --no-daemon --loglevel=debug --adaptive --ignore-cpuid-check
# Run your workload, and when done copy the output dumped on the screen
# to a file. For example if the output file is
# This output file is input to this script "thermald_log.txt"
# Then use:
# python thermald_debug_output_graph.py thermald_log.txt
#
# Prerequisites:
#    Python version 3.x or higher
#    gnuplot 5.0 or higher
#    python3-gnuplot 1.8 or higher
#    (Most of the distributions have these required packages. They may be called
#     gnuplot-py, python-gnuplot or python3-gnuplot, gnuplot-nox, ... )
#   In case Gnuplot is not present, the script will use pygnuplot. To do so, install
#   pygnuplot using pip install pygnuplot. In case can't install with pip, download
#   the pygnuplot.py file from https://pypi.org/project/py-gnuplot
#   To install use "python3 setup.py install". It may ask for pandas, install it
#   using "pip install pandas" or from distro install python3-pandas.

import sys
import re

use_pygnuplot = False

try:
    import Gnuplot
except ImportError:
    from pygnuplot import gnuplot
    use_pygnuplot = True

sensors = []
temperature = [[]]
temperature_count = []
max_range = 0

zones = []
trips = [[]]

# Parse file for temperature


def plot_temperature():
    output_png = "temperature.png"
    if use_pygnuplot:
        g_plot = gnuplot.Gnuplot()
    else:
        g_plot = Gnuplot.Gnuplot()

    g_plot('set ylabel "Degree C"')
    g_plot('set xlabel "Samples"')
    g_plot("set yrange [0:100]")
    g_plot("set key autotitle columnhead")
    g_plot('set title "Temperature plot"')
    g_plot("set term png size 1200, 600")
    g_plot('set output "' + output_png + '"')
    print("Plotting using gnuplot, the output file name is : ", output_png)
#    g_plot('plot "' + output_png + '" using 1:(column(n)) w lines title columnhead(n), for [n=2:50] "trips.csv" using 1:(column(n)) w dots title # columnhead(n)')
    g_plot(
        'plot for [n=2:12] "temperature.csv" u 1:(column(n)) w lines title columnhead(n), for [n=2:50] "trips.csv" u 1:(column(n)) w dots title columnhead(n)'
    )


def parse_trips(log_file):
    print("parsing trips from", log_file)
    log_fd = open(log_file, "r")
    data = log_fd.readlines()
    match_index = 0
    line_index = 0
    for line in data:
        x = re.search("ZONE DUMP BEGIN", line)
        if x:
            match_index = line_index
        line_index = line_index + 1

    for x in range(match_index, line_index):
        s = re.search("Zone.*:", data[x])
        if s:
            tokens = data[x].split()
            zone = tokens[2].split(",")
            zones.append(zone[0])

        y = re.search("ZONE DUMP END", data[x])
        if y:
            break

    index = 0
    trips.append([])
    # 3 for point to 3rd line starting from "ZONE DUMP BEGIN"
    for x in range(match_index + 3, line_index):
        s = re.search("passive temp:*", data[x])
        if s:
            tokens = data[x].split()
            trip = tokens[3].split(":")
            str1 = str(int(trip[1]) / 1000)
            trips[index].append(str1)

        z = re.search("Zone.*:", data[x])
        if z:
            index = index + 1
            trips.append([])

        y = re.search("ZONE DUMP END", data[x])
        if y:
            break

    max_index = index

    f_handle = open("trips.csv", "w")

    string_buffer = "index, "

    for x in range(len(zones)):
        id = 0
        for y in range(len(trips[x])):
            string_buffer += zones[x]
            string_buffer += "_trip_0"
            string_buffer += str(id)
            if x != (len(zones) - 1) and y != (len(trips) - 1):
                string_buffer += ", "
            id = id + 1

    string_buffer += "\n"
    f_handle.write(string_buffer)

    string_buffer = "0,"

    for i in range(max_range):
        string_buffer = str(i)
        string_buffer += ", "
        for x in range(len(zones)):
            for y in range(len(trips[x])):
                string_buffer += trips[x][y]
                if x != (len(zones) - 1) and y != (len(trips) - 1):
                    string_buffer += ", "

        string_buffer += "\n"
        f_handle.write(string_buffer)

    f_handle.close()

    f_handle.close()


def parse_temperature(log_file):
    # Parse file for temperature
    global max_range

    print("parsing temperature from", log_file)

    sensors.append("PL1")
    temperature_count.append(0)
    temp = []
    temperature.append(temp)

    log_fd = open(log_file, "r")
    data = log_fd.readlines()
    next_line = 0
    for line in data:
        next_line = next_line + 1
        x = re.search(".*wr:", line)
        if x:
            tokens = line.split()
            tokens = tokens[7].split(":")
            str1 = tokens[1]
            val = int(str1)
            val = val / 1000000
            if (val > 100):
                continue;
            str1 = str(val)
            temperature[0].append(str1)
            temperature_count[0] = temperature_count[0] + 1
            if max_range < temperature_count[0]:
                max_range = temperature_count[0]

        x = re.search("Sensor.*:temp", line)
        if not x:
            x = re.search("Sensor.*:power", line)

        if x:
            if "compare" in data[next_line]:
                continue
            tokens = line.split()
            skip = 0
            i = 0
            for sensor in sensors:
                if sensor == tokens[1]:
                    skip = 1
                    break
                i = i + 1
            if skip == 0:
                sensors.append(tokens[1])
                temperature_count.append(0)
                temp = []
                temperature.append(temp)

            str1 = str(int(tokens[3]) / 1000)

            temperature[i].append(str1)

            temperature_count[i] = temperature_count[i] + 1
            if max_range < temperature_count[i]:
                max_range = temperature_count[i]

    f_handle = open("temperature.csv", "w")

    last_val= []
    string_buffer = "index, "
    for x in range(len(sensors)):
        last_val.append(0)
        if x == len(sensors) - 1:
            string_buffer += sensors[x]
        else:
            string_buffer += sensors[x] + ", "

    string_buffer += "\n"
    f_handle.write(string_buffer)

    index = 0
    for y in range(max_range):
        string_buffer = str(index) + ", "
        index = index + 1
        for x in range(len(sensors)):
            if y < temperature_count[x]:
                last_val[x] = temperature[x][y]
                if x == len(sensors) - 1:
                    string_buffer += temperature[x][y]
                else:
                    string_buffer += temperature[x][y] + ", "

            else:
                string_buffer += last_val[x] + ", "

        string_buffer += "\n"
        f_handle.write(string_buffer)

    f_handle.close()

    log_fd.close()


# Main entry point
# Get the debug file name from command line
print("Number of arguments:", len(sys.argv), "arguments.")
print("Argument List:", str(sys.argv))

if len(sys.argv) < 2:
    print("Error: Need the logfile as argument")
    sys.exit()

# Parse temperature
parse_temperature(str(sys.argv[1]))
parse_trips(str(sys.argv[1]))
plot_temperature()



# Read gddv_test.json and generate gddv file
import json
import sys

data = b""
file_name = ""


def create_uintu64(value):
    global data
    header = b"\x04\x00\x00\x00"
    data = data + header + int(value).to_bytes(8, "little")


def create_string(value):
    global data
    header = b"\x08\x00\x00\x00"
    byte_array = value.encode()
    byte_array = byte_array + b'\x00'
    length = len(value) + 1
    length = length.to_bytes(8, "little")
    data = data + header + length + byte_array


def create_key(key, val_array):
    global data
    key_len = len(key) + 1
    key_flags = b"\x00\x00\x00\x00"
    key_length = int(key_len).to_bytes(4, "little")
    key_array = key.encode()
    key_array = key_array + b"\x00"

    val_type = b"\x00\x00\x00\x00"
    val_length = len(val_array).to_bytes(4, "little")

    data = data + key_flags + key_length + key_array + val_type + val_length + val_array


def create_header():
    global data
    data = b"\xE5\x1F\x0C\x00\x00\x00\x00\x01\x00\x00\x00\x00"


def create_psvt(binary_file):
    global data
    data = b""

    with open(file_name, "r") as gddv_test_file:
        gddv_data = json.load(gddv_test_file)
        print(json.dumps(gddv_data, indent=4))

        psvts = {}
        for key, value in gddv_data.items():
            if key == "PSVT":
                for index in range(len(gddv_data["PSVT"])):
                    psvts[index] = gddv_data["PSVT"][index]
                    print("index", index)
                    print("    name", psvts[index]["name"])
                    key_name = "/shared/tables/psvt/" + psvts[index]["name"]

                    table = psvts[index]["table"]
                    create_uintu64("2")
                    for i in range(len(table)):
                        print("table_index", i)
                        # create_uintu64("2")
                        print("    source:", table[i]["source"])
                        create_string(table[i]["source"])
                        print("    target:", table[i]["target"])
                        create_string(table[i]["target"])
                        print("    priority:", table[i]["priority"])
                        create_uintu64(table[i]["priority"])
                        print("    sample_peri od:", table[i]["sample_period"])
                        create_uintu64(table[i]["sample_period"])
                        print("    temperature:", table[i]["temperature"])
                        temp = int(table[i]["temperature"])
                        temp = temp * 10 + 2727
                        create_uintu64(str(temp))
                        print("    domain:", table[i]["domain"])
                        create_uintu64(table[i]["domain"])
                        print("    control_knob:", table[i]["control_knob"])
                        create_uintu64(table[i]["control_knob"])
                        if table[i]["limit"] == "MIN" or table[i]["limit"] == "MAX":
                            create_string(table[i]["limit"])
                        else:
                            create_uintu64(table[i]["limit"])
                        print("    step_size:", table[i]["step_size"])
                        create_uintu64(table[i]["step_size"])
                        print("    limit_coeff:", table[i]["limit_coeff"])
                        create_uintu64(table[i]["limit_coeff"])
                        print("    unlimit_coeff:", table[i]["unlimit_coeff"])
                        create_uintu64(table[i]["unlimit_coeff"])
                        data = (
                            data + b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                        )
                        print("(", data, ")")

                    print("[", data, "]")
                    psvt_entry = data
                    data = b""
                    create_key(key_name, psvt_entry)
                    binary_file.write(data)
                    data = b""


def parse_psvt_json(binary_file):
    print(binary_file)
    create_psvt(binary_file)


def create_gddv():
    print("create gddv")
    with open("gddv.bin", "wb") as binary_file:
        create_header()
        binary_file.write(data)
        parse_psvt_json(binary_file)


# main entry point
# Get the config file name from command line
print("Number of arguments:", len(sys.argv), "arguments.")
print("Argument List:", str(sys.argv))

if len(sys.argv) < 2:
    print("Error: Need the Json config file as argument")
    sys.exit()

file_name = str(sys.argv[1])
create_gddv()



# -*- coding: utf-8 -*-

# script to dump thermal config from the running thermald via dbus
# The caller must be in "power" group

import dbus
import os

def dump_thermal_config():

    if os.path.exists(file_name):
        os.remove(file_name)

    f_handle = open(file_name, 'a')

    system_bus = dbus.SystemBus()

    thd = system_bus.get_object('org.freedesktop.thermald',
                                '/org/freedesktop/thermald')
    thd_intf = dbus.Interface(thd,
                              dbus_interface='org.freedesktop.thermald')

    zone_count = thd_intf.GetZoneCount()
    string_buffer = 'zones count:' + str(zone_count) + '\n'
    f_handle.write(string_buffer)

    sensor_count = thd_intf.GetSensorCount()
    string_buffer = 'sensors count:' + str(sensor_count) + '\n'
    f_handle.write(string_buffer)
    f_handle.write('\n')

    for i in range(zone_count):
        zone = thd_intf.GetZoneInformation(i)

        string_buffer = 'zone id' + ': ' + str(i) + '''

'''
        f_handle.write(string_buffer)

        string_buffer = '\tzone name' + ': ' + zone[0] + '\n'
        f_handle.write(string_buffer)

        string_buffer = '\tSensor Count' + ': ' + str(zone[1]) + '\n'
        f_handle.write(string_buffer)

        for j in range(zone[1]):
            sensor = thd_intf.GetZoneSensorAtIndex(i, j)
            string_buffer = '\t\t sensor: ' + sensor + '\n'
            f_handle.write(string_buffer)

        string_buffer = '\tTrip Count' + ': ' + str(zone[2]) + '\n'
        f_handle.write(string_buffer)

        for j in range(zone[2]):
            string_buffer = '\tTrip id' + ': ' + str(j) + '\n'
            f_handle.write(string_buffer)

            trip = thd_intf.GetZoneTripAtIndex(i, j)

            string_buffer = '\t\t\t trip temp' + ': ' + str(trip[0]) \
                + '\n'
            f_handle.write(string_buffer)

            string_buffer = '\t\t\t trip type' + ': ' + str(trip[1]) \
                + '\n'
            f_handle.write(string_buffer)

            string_buffer = '\t\t\t sensor id' + ': ' + str(trip[2]) \
                + '\n'
            f_handle.write(string_buffer)

        string_buffer = '''

'''
        f_handle.write(string_buffer)

    cdev_count = thd_intf.GetCdevCount()
    string_buffer = 'cdev count:' + str(cdev_count) + '\n'
    f_handle.write(string_buffer)
    f_handle.write('\n')

    for i in range(cdev_count):
        string_buffer = 'cdev id' + ': ' + str(i) + '\n'
        f_handle.write(string_buffer)

        cdev = thd_intf.GetCdevInformation(i)
        string_buffer = '\t\t type' + ': ' + str(cdev[0]) + '\n'
        f_handle.write(string_buffer)

        string_buffer = '\t\t min_state' + ': ' + str(cdev[1]) + '\n'
        f_handle.write(string_buffer)

        string_buffer = '\t\t max_state' + ': ' + str(cdev[2]) + '\n'
        f_handle.write(string_buffer)

        string_buffer = '\t\t current_state' + ': ' + str(cdev[3]) \
            + '\n'
        f_handle.write(string_buffer)

    f_handle.close()


def remove_space(string):
    return string.replace(' ', '')


if __name__ == '__main__':
    product_name = '/sys/class/dmi/id/product_name'

    try:
        file_name = open(product_name, 'r').read()
    except:
        print ('Error opening ', file_name)
        sys.exit(2)

    file_name = '/tmp/' + file_name.strip()
    file_name = remove_space(file_name) + '.config'

    print ('File Name to dump config:', file_name)

    dump_thermal_config()




#!/usr/bin/python
# -*- coding: utf-8 -*-

# script to dump temperature samples every 10 seconds
# The caller must be in "power" group to call the script

import dbus
import os
import time


def dump_temperature():

    if os.path.exists(file_name):
        os.remove(file_name)

    f_handle = open(file_name, 'a')

    system_bus = dbus.SystemBus()

    thd = system_bus.get_object('org.freedesktop.thermald',
                                '/org/freedesktop/thermald')
    thd_intf = dbus.Interface(thd,
                              dbus_interface='org.freedesktop.thermald')

    sensor_count = thd_intf.GetSensorCount()
    string_buffer = 'sensor count:' + str(sensor_count) + '\n'
    f_handle.write(string_buffer)
    f_handle.write('\n')

    string_buffer = ''
    for i in range(sensor_count):
        sensor_info = thd_intf.GetSensorInformation(i)
        string_buffer = string_buffer + sensor_info[0] + ','

    f_handle.write(string_buffer)
    f_handle.write('\n')

    string_buffer = ''

    f_lock = open('/tmp/thermal_temp_dump.lock', 'a')
    f_lock.close()

    while True:
        if os.path.exists('/tmp/thermal_temp_dump.lock') == False:
            break
        string_buffer = ''
        for i in range(sensor_count):
            sensor = thd_intf.GetSensorTemperature(i)
            string_buffer = string_buffer + str(sensor / 1000) + ','
        f_handle.write(string_buffer)
        f_handle.write('\n')
        time.sleep(10)

    f_handle.close()


def remove_spaces(string):
    return string.replace(' ', '')


if __name__ == '__main__':
    product_name = '/sys/class/dmi/id/product_name'

    try:
        file_name = open(product_name, 'r').read()
    except:
        print ('Error opening ', file_name)
        sys.exit(2)

    file_name = '/tmp/' + file_name.strip()
    file_name = remove_spaces(file_name) + '.csv'

    print ('File Name to dump config:', file_name)
    print ('Before starting this script, do the following steps')
    print ('sudo touch /var/run/thermald/debug_mode')
    print ('sudo systemctl restart thermald')
    print ('This will also dump power data from rapl as samples')
    print ('This script will loop forever till user deletes file')
    print ('/tmp/thermal_temp_dump.lock')

    dump_temperature()
